#![no_std]
#![no_main]

mod touch;

use cortex_m_rt::entry;

use panic_halt as _;

use stm32f0xx_hal as hal;

use hal::adc::Adc;
use hal::pac::Peripherals;
use hal::prelude::*;

use touch::Touch;

#[entry]
fn main() -> ! {
    let mut p = Peripherals::take().unwrap();

    let touch = Touch::new(p.SYSCFG, p.EXTI, &mut p.RCC);

    let mut rcc = p
        .RCC
        .configure()
        .hsi48()
        .enable_crs(p.CRS)
        .sysclk(48.mhz())
        .pclk(24.mhz())
        .freeze(&mut p.FLASH);

    let gpioa = p.GPIOA.split(&mut rcc);
    let gpiob = p.GPIOB.split(&mut rcc);
    let gpioc = p.GPIOC.split(&mut rcc);
    let gpiof = p.GPIOF.split(&mut rcc);

    #[allow(unused)]
    let (
        mut adc3,   // ADC_IN0
        mut adc2,   // ADC_IN1
        mut adc1,   // ADC_IN2
        mut adc8,   // ADC_IN3
        mut adc7,   // ADC_IN4
        mut adc6,   // ADC_IN5
        mut adc5,   // ADC_IN6
        mut adc4,   // ADC_IN7
        pwm6,       // TIM1_CH1
        tx,         // USART1_TX
        rx,         // USART1_RX
        usb_dm,     // USB_DM
        usb_dp,     // USB_DP
        mut dir5,   // PA13
        mut dir4,   // PA14
        mut dir3,   // PA15
        mut touch4, // PB0
        mut touch5, // PB1
        mut touch6, // PB2
        pwm5,       // TIM2_CH2
        pwm4,       // TIM3_CH1
        pwm3,       // TIM3_CH2
        mut dir2,   // PB6
        mut dir1,   // PB7
        pwm1,       // TIM16_CH1
        pwm2,       // TIM17_CH1
        mut touch7, // PB10
        mut touch8, // PB11
        mut dir8,   // PB12
        mut dir7,   // PB13
        pwm8,       // TIM15_CH1
        pwm7,       // TIM15_CH2
        mut touch1, // PC13
        mut touch2, // PC14
        mut touch3, // PC15
        mut nsleep, // PF0
        mut dir6,   // PF1
    ) = cortex_m::interrupt::free(|cs| {
        (
            gpioa.pa0.into_analog(cs),
            gpioa.pa1.into_analog(cs),
            gpioa.pa2.into_analog(cs),
            gpioa.pa3.into_analog(cs),
            gpioa.pa4.into_analog(cs),
            gpioa.pa5.into_analog(cs),
            gpioa.pa6.into_analog(cs),
            gpioa.pa7.into_analog(cs),
            gpioa.pa8.into_alternate_af2(cs),
            gpioa.pa9.into_alternate_af1(cs),
            gpioa.pa10.into_alternate_af1(cs),
            gpioa.pa11,
            gpioa.pa12,
            gpioa.pa13.into_push_pull_output(cs),
            gpioa.pa14.into_push_pull_output(cs),
            gpioa.pa15.into_push_pull_output(cs),
            gpiob.pb0.into_open_drain_output(cs),
            gpiob.pb1.into_open_drain_output(cs),
            gpiob.pb2.into_open_drain_output(cs),
            gpiob.pb3.into_alternate_af2(cs),
            gpiob.pb4.into_alternate_af1(cs),
            gpiob.pb5.into_alternate_af1(cs),
            gpiob.pb6.into_push_pull_output(cs),
            gpiob.pb7.into_push_pull_output(cs),
            gpiob.pb8.into_alternate_af2(cs),
            gpiob.pb9.into_alternate_af2(cs),
            gpiob.pb10.into_open_drain_output(cs),
            gpiob.pb11.into_open_drain_output(cs),
            gpiob.pb12.into_push_pull_output(cs),
            gpiob.pb13.into_push_pull_output(cs),
            gpiob.pb14.into_alternate_af1(cs),
            gpiob.pb15.into_alternate_af1(cs),
            gpioc.pc13.into_open_drain_output(cs),
            gpioc.pc14.into_open_drain_output(cs),
            gpioc.pc15.into_open_drain_output(cs),
            gpiof.pf0.into_push_pull_output(cs),
            gpiof.pf1.into_push_pull_output(cs),
        )
    });

    let mut _adc = Adc::new(p.ADC, &mut rcc);

    touch.setup();

    loop {}
}
